# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup.items import filesystems


class File(filesystems.Filesystem):


    def __init__(self, conf):
        super(File, self).__init__(conf)


    def type(self):
        return 'filesystems/file'


    def run(self):
        super(File, self).run(with_intermediate_file=True)


    def build_dump_command(self):
        return 'cat %s' % self.path


    def build_restore_command(self, backup_filename):
        return 'cp -p %s/%s %s' % (self.output_directory,
                                   backup_filename,
                                   self.path)
